package javax.swing.text.html;

import java.io.File;
import javax.swing.JFileChooser;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.util.BitSet;
import javax.swing.text.PlainDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.ElementIterator;
import java.net.URLEncoder;
import java.awt.Point;
import javax.swing.SwingUtilities;
import java.io.IOException;
import javax.swing.event.HyperlinkEvent;
import javax.swing.JEditorPane;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListDataListener;
import javax.swing.event.DocumentListener;
import javax.swing.text.AbstractDocument;
import javax.swing.AbstractListModel;
import javax.swing.DefaultButtonModel;
import javax.swing.ButtonModel;
import java.net.MalformedURLException;
import javax.swing.ImageIcon;
import java.net.URL;
import javax.swing.JCheckBox;
import javax.swing.JToggleButton;
import javax.swing.JRadioButton;
import javax.swing.JPasswordField;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.UIManager;
import javax.swing.JTextField;
import javax.swing.JComponent;
import javax.swing.text.AttributeSet;
import javax.swing.ListSelectionModel;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.JComboBox;
import javax.swing.ComboBoxModel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.text.Document;
import javax.swing.text.StyleConstants;
import java.awt.Component;
import javax.swing.text.Element;
import java.awt.event.ActionListener;
import javax.swing.text.ComponentView;

public class FormView extends ComponentView implements ActionListener
{
    @Deprecated
    public static final String SUBMIT;
    @Deprecated
    public static final String RESET;
    static final String PostDataProperty = "javax.swing.JEditorPane.postdata";
    private short maxIsPreferred;
    
    public FormView(Element elem) {
        super(elem);
    }
    
    @Override
    protected Component createComponent() {
        AttributeSet attributes = this.getElement().getAttributes();
        HTML.Tag tag = (HTML.Tag)attributes.getAttribute(StyleConstants.NameAttribute);
        JComponent inputComponent = null;
        Object attribute = attributes.getAttribute(StyleConstants.ModelAttribute);
        this.removeStaleListenerForModel(attribute);
        if (tag == HTML.Tag.INPUT) {
            inputComponent = this.createInputComponent(attributes, attribute);
        }
        else if (tag == HTML.Tag.SELECT) {
            if ((attribute instanceof OptionListModel)) {
                JList view2 = new JList((ListModel)attribute);
                view2.setVisibleRowCount(HTML.getIntegerAttributeValue(attributes, HTML.Attribute.SIZE, 1));
                view2.setSelectionModel((ListSelectionModel)attribute);
                inputComponent = new JScrollPane(view2);
            }
            else {
                inputComponent = new JComboBox((ComboBoxModel)attribute);
                this.maxIsPreferred = 3;
            }
        }
        else if (tag == HTML.Tag.TEXTAREA) {
            JTextArea view = new JTextArea((Document)attribute);
            view.setRows(HTML.getIntegerAttributeValue(attributes, HTML.Attribute.ROWS, 1));
            int integerAttributeValue = HTML.getIntegerAttributeValue(attributes, HTML.Attribute.COLS, 20);
            this.maxIsPreferred = 3;
            view.setColumns(integerAttributeValue);
            inputComponent = new JScrollPane(view, 22, 32);
        }
        if (inputComponent != null) {
            inputComponent.setAlignmentY(1.0f);
            return inputComponent;
        }
        return inputComponent;
    }
    
    private JComponent createInputComponent(AttributeSet set, Object o) {
        Object o2 = null;
        String s = (String)set.getAttribute(HTML.Attribute.TYPE);
        if (s.equals("submit") || s.equals("reset")) {
            String text = (String)set.getAttribute(HTML.Attribute.VALUE);
            if (text == null) {
                if (s.equals("submit")) {
                    text = UIManager.getString("FormView.submitButtonText");
                }
                else {
                    text = UIManager.getString("FormView.resetButtonText");
                }
            }
            JButton o3 = new JButton(text);
            if (o != null) {
                o3.setModel((ButtonModel)o);
                o3.addActionListener(this);
            }
            o2 = o3;
            this.maxIsPreferred = 3;
        }
        else if (s.equals("image")) {
            String s2 = (String)set.getAttribute(HTML.Attribute.SRC);
            JButton button;
            try {
                button = new JButton(new ImageIcon(new URL(((HTMLDocument)this.getElement().getDocument()).getBase(), s2)));
            }
            catch (MalformedURLException ex) {
                button = new JButton(s2);
            }
            if (o != null) {
                button.setModel((ButtonModel)o);
                button.addMouseListener(new FormView.MouseEventListener());
            }
            o2 = button;
            this.maxIsPreferred = 3;
        }
        else if (s.equals("checkbox")) {
            o2 = new JCheckBox();
            if (o != null) {
                ((JCheckBox)o2).setModel((JToggleButton.ToggleButtonModel)o);
            }
            this.maxIsPreferred = 3;
        }
        else if (s.equals("radio")) {
            o2 = new JRadioButton();
            if (o != null) {
                ((JRadioButton)o2).setModel((JToggleButton.ToggleButtonModel)o);
            }
            this.maxIsPreferred = 3;
        }
        else if (s.equals("text")) {
            int integerAttributeValue3 = HTML.getIntegerAttributeValue(set, HTML.Attribute.SIZE, -1);
            JTextField textField;
            if (integerAttributeValue3 > 0) {
                textField = new JTextField();
                textField.setColumns(integerAttributeValue3);
            }
            else {
                textField = new JTextField();
                textField.setColumns(20);
            }
            o2 = textField;
            if (o != null) {
                textField.setDocument((Document)o);
            }
            textField.addActionListener(this);
            this.maxIsPreferred = 3;
        }
        else if (s.equals("password")) {
            JPasswordField passwordField = o2 = new JPasswordField();
            if (o != null) {
                passwordField.setDocument((Document)o);
            }
            int integerAttributeValue2 = HTML.getIntegerAttributeValue(set, HTML.Attribute.SIZE, -1);
            passwordField.setColumns((integerAttributeValue2 > 0) ? integerAttributeValue2 : 20);
            passwordField.addActionListener(this);
            this.maxIsPreferred = 3;
        }
        else if (s.equals("file")) {
            JTextField comp = new JTextField();
            if (o != null) {
                comp.setDocument((Document)o);
            }
            int integerAttributeValue = HTML.getIntegerAttributeValue(set, HTML.Attribute.SIZE, -1);
            comp.setColumns((integerAttributeValue > 0) ? integerAttributeValue : 20);
            JButton comp2 = new JButton(UIManager.getString("FormView.browseFileButtonText"));
            Box horizontalBox = Box.createHorizontalBox();
            horizontalBox.add(comp);
            horizontalBox.add(Box.createHorizontalStrut(5));
            horizontalBox.add(comp2);
            comp2.addActionListener(new FormView.BrowseFileAction(set, (Document)o));
            o2 = horizontalBox;
            this.maxIsPreferred = 3;
        }
        return o2;
    }
    
    private void removeStaleListenerForModel(Object o) {
        if ((o instanceof DefaultButtonModel)) {
            DefaultButtonModel defaultButtonModel = (DefaultButtonModel)o;
            String s5 = "javax.swing.AbstractButton$Handler";
            for (ActionListener m : defaultButtonModel.getActionListeners()) {
                if (s5.equals(m.getClass().getName())) {
                    defaultButtonModel.removeActionListener(m);
                }
            }
            for (ChangeListener l2 : defaultButtonModel.getChangeListeners()) {
                if (s5.equals(l2.getClass().getName())) {
                    defaultButtonModel.removeChangeListener(l2);
                }
            }
            for (ItemListener l3 : defaultButtonModel.getItemListeners()) {
                if (s5.equals(l3.getClass().getName())) {
                    defaultButtonModel.removeItemListener(l3);
                }
            }
        }
        else if ((o instanceof AbstractListModel)) {
            AbstractListModel<Object> abstractListModel = (AbstractListModel)o;
            String s3 = "javax.swing.plaf.basic.BasicListUI$Handler";
            String s4 = "javax.swing.plaf.basic.BasicComboBoxUI$Handler";
            for (ListDataListener l : abstractListModel.getListDataListeners()) {
                if (s3.equals(l.getClass().getName()) || s4.equals(l.getClass().getName())) {
                    abstractListModel.removeListDataListener(l);
                }
            }
        }
        else if ((o instanceof AbstractDocument)) {
            String s = "javax.swing.plaf.basic.BasicTextUI$UpdateHandler";
            String s2 = "javax.swing.text.DefaultCaret$Handler";
            AbstractDocument abstractDocument = (AbstractDocument)o;
            for (DocumentListener listener : abstractDocument.getDocumentListeners()) {
                if (s.equals(listener.getClass().getName()) || s2.equals(listener.getClass().getName())) {
                    abstractDocument.removeDocumentListener(listener);
                }
            }
        }
    }
    
    @Override
    public float getMaximumSpan(int axis) {
        switch (axis) {
            case 0: {
                if ((this.maxIsPreferred & 0x1) == 0x1) {
                    super.getMaximumSpan(axis);
                    return this.getPreferredSpan(axis);
                }
                return super.getMaximumSpan(axis);
            }
            case 1: {
                if ((this.maxIsPreferred & 0x2) == 0x2) {
                    super.getMaximumSpan(axis);
                    return this.getPreferredSpan(axis);
                }
                return super.getMaximumSpan(axis);
            }
            default: {
                return super.getMaximumSpan(axis);
            }
        }
    }
    
    @Override
    public void actionPerformed(ActionEvent actionEvent) {
        Element element = this.getElement();
        StringBuilder sb = new StringBuilder();
        HTMLDocument htmlDocument = (HTMLDocument)this.getDocument();
        String s = (String)element.getAttributes().getAttribute(HTML.Attribute.TYPE);
        if (s.equals("submit")) {
            this.getFormData(sb);
            this.submitData(sb.toString());
        }
        else if (s.equals("reset")) {
            this.resetForm();
        }
        else if (s.equals("text") || s.equals("password")) {
            if (this.isLastTextOrPasswordField()) {
                this.getFormData(sb);
                this.submitData(sb.toString());
            }
            else {
                this.getComponent().transferFocus();
            }
        }
    }
    
    protected void submitData(String s) {
        Element formElement = this.getFormElement();
        AttributeSet attributes = formElement.getAttributes();
        HTMLDocument htmlDocument = (HTMLDocument)formElement.getDocument();
        URL base = htmlDocument.getBase();
        String s2 = (String)attributes.getAttribute(HTML.Attribute.TARGET);
        if (s2 == null) {
            s2 = "_self";
        }
        String s3 = (String)attributes.getAttribute(HTML.Attribute.METHOD);
        if (s3 == null) {
            s3 = "GET";
        }
        int equals = s3.toLowerCase().equals("post");
        if (!(equals == 0)) {
            this.storePostData(htmlDocument, s2, s);
        }
        String spec = (String)attributes.getAttribute(HTML.Attribute.ACTION);
        final Object obj;
        try {
            obj = ((spec == null) ? new URL(base.getProtocol(), base.getHost(), base.getPort(), base.getFile()) : new URL(base, spec));
            if (equals == 0) {
                obj = new URL(obj + "?" + s.toString());
            }
        }
        catch (MalformedURLException ex) {
            obj = null;
        }
        final JEditorPane editorPane = (JEditorPane)this.getContainer();
        HTMLEditorKit htmlEditorKit = (HTMLEditorKit)editorPane.getEditorKit();
        final Object o = null;
        if (!htmlEditorKit.isAutoFormSubmission() || htmlDocument.isFrameDocument()) {
            o = new FormSubmitEvent(this, HyperlinkEvent.EventType.ACTIVATED, obj, formElement, s2, equals ? FormSubmitEvent.MethodType.POST : FormSubmitEvent.MethodType.GET, s);
        }
        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                if (o != null) {
                    editorPane.fireHyperlinkUpdate(o);
                }
                else {
                    try {
                        editorPane.setPage(obj);
                    }
                    catch (IOException ex) {
                        UIManager.getLookAndFeel().provideErrorFeedback(editorPane);
                    }
                }
            }
        });
    }
    
    private void storePostData(HTMLDocument document, String str, String s) {
        String string = "javax.swing.JEditorPane.postdata";
        if (document.isFrameDocument()) {
            JEditorPane outermostJEditorPane = ((FrameView.FrameEditorPane)this.getContainer()).getFrameView().getOutermostJEditorPane();
            if (outermostJEditorPane != null) {
                document = outermostJEditorPane.getDocument();
                string = string + "." + str;
            }
        }
        document.putProperty(string, s);
    }
    
    protected void imageSubmit(String str) {
        StringBuilder sb = new StringBuilder();
        HTMLDocument htmlDocument = (HTMLDocument)this.getElement().getDocument();
        this.getFormData(sb);
        if (sb.length() > 0) {
            sb.append('&');
        }
        sb.append(str);
        this.submitData(sb.toString());
    }
    
    private String getImageData(Point point) {
        String string = point.x + ":" + point.y;
        int index = string.indexOf(58);
        String substring = string.substring(0, index);
        String substring2 = string.substring(++index);
        String s = (String)this.getElement().getAttributes().getAttribute(HTML.Attribute.NAME);
        if (s == null || s.equals("")) {
            return "x=" + substring + "&y=" + substring2;
        }
        String encode = URLEncoder.encode(s);
        return encode + ".x=" + substring + "&" + encode + ".y=" + substring2;
    }
    
    private Element getFormElement() {
        Element element = this.getElement();
        while (element != null) {
            if (element.getAttributes().getAttribute(StyleConstants.NameAttribute) == HTML.Tag.FORM) {
                return element;
            }
            element = element.getParentElement();
        }
        return null;
    }
    
    private void getFormData(StringBuilder sb) {
        Element formElement = this.getFormElement();
        if (formElement != null) {
            Element next;
            while ((next = new ElementIterator(formElement).next()) != null) {
                if (!this.isControl(next)) {
                    continue;
                }
                String s = (String)next.getAttributes().getAttribute(HTML.Attribute.TYPE);
                if (s != null && s.equals("submit") && next != this.getElement()) {
                    continue;
                }
                if (s == null || !s.equals("image")) {
                    this.loadElementDataIntoBuffer(next, sb);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    private void loadElementDataIntoBuffer(Element element, StringBuilder sb) {
        AttributeSet attributes = element.getAttributes();
        String s = (String)attributes.getAttribute(HTML.Attribute.NAME);
        if (s == null) {
            return;
        }
        String s2 = null;
        HTML.Tag tag = (HTML.Tag)element.getAttributes().getAttribute(StyleConstants.NameAttribute);
        if (tag == HTML.Tag.INPUT) {
            s2 = this.getInputElementData(attributes);
        }
        else if (tag == HTML.Tag.TEXTAREA) {
            s2 = this.getTextAreaData(attributes);
        }
        else if (tag == HTML.Tag.SELECT) {
            this.loadSelectData(attributes, sb);
        }
        if (s != null && s2 != null) {
            this.appendBuffer(sb, s, s2);
        }
        return;
    }
    
    private String getInputElementData(AttributeSet set) {
        Object attribute = set.getAttribute(StyleConstants.ModelAttribute);
        String s = (String)set.getAttribute(HTML.Attribute.TYPE);
        String text = null;
        if (s.equals("text") || s.equals("password")) {
            Document document2 = (Document)attribute;
            try {
                text = document2.getText(0, document2.getLength());
            }
            catch (BadLocationException ex2) {
                text = null;
            }
        }
        else if (s.equals("submit") || s.equals("hidden")) {
            text = (String)set.getAttribute(HTML.Attribute.VALUE);
            if (text == null) {
                text = "";
            }
        }
        else if (s.equals("radio") || s.equals("checkbox")) {
            if (((ButtonModel)attribute).isSelected()) {
                text = (String)set.getAttribute(HTML.Attribute.VALUE);
                if (text == null) {
                    text = "on";
                }
            }
        }
        else if (s.equals("file")) {
            Document document = (Document)attribute;
            String text2;
            try {
                text2 = document.getText(0, document.getLength());
            }
            catch (BadLocationException ex) {
                text2 = null;
            }
            if (text2 != null && text2.length() > 0) {
                text = text2;
            }
        }
        return text;
    }
    
    private String getTextAreaData(AttributeSet set) {
        Document document = (Document)set.getAttribute(StyleConstants.ModelAttribute);
        try {
            return document.getText(0, document.getLength());
        }
        catch (BadLocationException ex) {
            return null;
        }
    }
    
    private void loadSelectData(AttributeSet set, StringBuilder sb) {
        String s = (String)set.getAttribute(HTML.Attribute.NAME);
        if (s == null) {
            return;
        }
        Object attribute = set.getAttribute(StyleConstants.ModelAttribute);
        if ((attribute instanceof OptionListModel)) {
            OptionListModel<Object> optionListModel = (OptionListModel)attribute;
            for (int i = 0; i < optionListModel.getSize(); ++i) {
                if (optionListModel.isSelectedIndex(i)) {
                    this.appendBuffer(sb, s, ((Option)optionListModel.getElementAt(i)).getValue());
                }
            }
        }
        else if ((attribute instanceof ComboBoxModel)) {
            Option option = (Option)((ComboBoxModel)attribute).getSelectedItem();
            if (option != null) {
                this.appendBuffer(sb, s, option.getValue());
            }
        }
        return;
    }
    
    private void appendBuffer(StringBuilder sb, String s, String s2) {
        if (sb.length() > 0) {
            sb.append('&');
        }
        sb.append(URLEncoder.encode(s));
        sb.append('=');
        sb.append(URLEncoder.encode(s2));
    }
    
    private boolean isControl(Element element) {
        return element.isLeaf();
    }
    
    boolean isLastTextOrPasswordField() {
        Element formElement = this.getFormElement();
        Element element = this.getElement();
        if (formElement != null) {
            ElementIterator elementIterator = new ElementIterator(formElement);
            boolean n = false;
            Element next;
            while ((next = elementIterator.next()) != null) {
                if (next == element) {
                    n = true;
                }
                else {
                    if (n == 0 || !this.isControl(next)) {
                        continue;
                    }
                    AttributeSet attributes = next.getAttributes();
                    if (HTMLDocument.matchNameAttribute(attributes, HTML.Tag.INPUT)) {
                        String s = (String)attributes.getAttribute(HTML.Attribute.TYPE);
                        if ("text".equals(s) || "password".equals(s)) {
                            return false;
                        }
                        continue;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return true;
    }
    
    void resetForm() {
        Element formElement = this.getFormElement();
        if (formElement != null) {
            Element next;
            while ((next = new ElementIterator(formElement).next()) != null) {
                if (!this.isControl(next)) {
                    continue;
                }
                AttributeSet attributes = next.getAttributes();
                Object attribute = attributes.getAttribute(StyleConstants.ModelAttribute);
                if ((attribute instanceof TextAreaDocument)) {
                    ((TextAreaDocument)attribute).reset();
                }
                else if ((attribute instanceof PlainDocument)) {
                    try {
                        PlainDocument plainDocument = (PlainDocument)attribute;
                        plainDocument.remove(0, plainDocument.getLength());
                        if (HTMLDocument.matchNameAttribute(attributes, HTML.Tag.INPUT)) {
                            String str = (String)attributes.getAttribute(HTML.Attribute.VALUE);
                            if (str != null) {
                                plainDocument.insertString(0, str, null);
                            }
                            else {
                                continue;
                            }
                        }
                        else {
                            continue;
                        }
                    }
                    catch (BadLocationException ex) {}
                }
                else if ((attribute instanceof OptionListModel)) {
                    OptionListModel<Object> optionListModel = (OptionListModel)attribute;
                    for (int size = optionListModel.getSize(), i = 0; i < size; ++i) {
                        optionListModel.removeIndexInterval(i, i);
                    }
                    BitSet initialSelection2 = optionListModel.getInitialSelection();
                    for (int j = 0; j < initialSelection2.size(); ++j) {
                        if (initialSelection2.get(j)) {
                            optionListModel.addSelectionInterval(j, j);
                        }
                    }
                }
                else if ((attribute instanceof OptionComboBoxModel)) {
                    OptionComboBoxModel<Object> optionComboBoxModel = (OptionComboBoxModel)attribute;
                    Option initialSelection = optionComboBoxModel.getInitialSelection();
                    if (initialSelection != null) {
                        optionComboBoxModel.setSelectedItem(initialSelection);
                    }
                    else {
                        continue;
                    }
                }
                else if ((attribute instanceof JToggleButton.ToggleButtonModel)) {
                    ((JToggleButton.ToggleButtonModel)attribute).setSelected((String)attributes.getAttribute(HTML.Attribute.CHECKED) != null);
                }
                else {
                    continue;
                }
            }
        }
    }
    
    static String access$000(FormView formView, Point point) {
        return formView.getImageData(point);
    }
    
    static {
        SUBMIT = new String("Submit Query");
        RESET = new String("Reset");
    }
    
    protected class MouseEventListener extends MouseAdapter
    {
        @Override
        public void mouseReleased(MouseEvent mouseEvent) {
            FormView.this.imageSubmit(FormView.access$000(FormView.this, mouseEvent.getPoint()));
        }
    }
    
    private class BrowseFileAction implements ActionListener
    {
        private AttributeSet attrs;
        private Document model;
        
        BrowseFileAction(AttributeSet attrs, Document model) {
            this.attrs = attrs;
            this.model = model;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JFileChooser fileChooser = new JFileChooser();
            fileChooser.setMultiSelectionEnabled(false);
            if (fileChooser.showOpenDialog(FormView.this.getContainer()) == 0) {
                File selectedFile = fileChooser.getSelectedFile();
                if (selectedFile != null) {
                    try {
                        if (this.model.getLength() > 0) {
                            this.model.remove(0, this.model.getLength());
                        }
                        this.model.insertString(0, selectedFile.getPath(), null);
                    }
                    catch (BadLocationException ex) {}
                }
            }
        }
    }
}
